; ***********************************************************************
; *	                PIC12F675					*
; *                    ----\/----					*
; *      V+---------- |1        8| GND					*
; *            ---GP5 |2        7| GP0---			        *
; *            ---GP4 |3        6| GP1---				*
; *   RESET    ---GP3 |4        5| GP2---			        *
; *                    ----------					*
; ***********************************************************************

    processor 12F675
    #include <P12F675.INC>
    __config 0x3F84
;   _CPD_OFF & _CP_OFF & _BODEN_OFF & _MCLRE_OFF & _PWRTE_ON & _WDT_OFF 
;   & _INTRC_OSC_NOCLKOUT 

; __CONFIG 031F4H                  
;   _CPD_OFF & _CP_OFF & _BODEN_ON & _MCLRE_ON & _PWRTE_ON & _WDT_OFF
;   &_FOSC_INTRCIO 

;*********************************************************
   		cblock	0x20
	PAD_FLAG 
	tmr1var1	; length of 1 dot
	tmr1var2        ; length of 1 dash
	tmr1var3        ; length common register   
	counter
    		endc
;*********************************************************
    Org 0x0000

    CLRF 	tmr1var3
    CLRF 	PAD_FLAG
    MOVLW 	0x1E     	;   b'00011110' 
    MOVWF 	GPIO      	;   GPIO initial state
    MOVLW 	0x1F           	;   b'00011111' 
    MOVWF 	CMCON          	;   comparator OFF / CIN+ on GP0 /
    MOVLW 	0x30           	;   b'00110000' 
    MOVWF 	T1CON          	;

    BSF 	STATUS,RP0   	; 	Bank1
    MOVLW 	0x0F           	;   b'00001111' GPIO,4 and GPIO,5 OUT
    MOVWF 	TRISIO       	;
    CLRF 	OPTION_REG    	;
    MOVLW 	0x06           	;   b'00000110' PULL-UP on GPIO1,GPIO2
    MOVWF 	WPU           
    CLRF 	INTCON          ;   interrupts main register
    call	0x3FF
    MOVWF 	OSCCAL
    CLRF 	ANSEL		;   digital I/O
    MOVLW 	0x8F           	;   b'10001111' = 3,6V
    MOVWF 	VRCON		;   comparator voltage reference value 
    BCF 	STATUS,RP0      ;  	Bank0

    BCF 	ADCON0,0    	;   A/D control register OFF

    CALL 	delay1

;*********************************************************

start

    BTFSS 	GPIO,2         	; Check Paddle 2
    GOTO 	paddle2
    BTFSS 	GPIO,1         	; Check Paddle 1
    GOTO 	paddle1
    CALL 	chk_speed     	; Check speed
    
    GOTO 	start

;************************* DOT ***************************

paddle2
    BSF 	GPIO,5         	; Set OUT ON
    MOVF 	tmr1var1,W      ; load length
    MOVWF 	tmr1var3	; of 1 dot
    BCF 	PAD_FLAG,0	; reset memory flag
loop21
    BSF 	GPIO,4       	; Set AUDIO ON
    CALL 	delay1		; 0,65ms : half period for 750Hz
    CALL 	mem_pad1	; check another paddle while ON
    BCF 	GPIO,4         	; Set AUDIO OFF
    CALL 	delay1		; 0,65ms : half period for 750Hz
    CALL 	mem_pad1	; check another paddle while ON
    DECFSZ 	tmr1var3,F      ; check if length exhausted
    GOTO 	loop21          ; cycling if not
    BCF 	GPIO,5         	; Set OUT OFF
    NOP
    MOVF 	tmr1var1,W	; load length
    MOVWF 	tmr1var3	; of pause = 1 dot
    NOP
loop22
    NOP				; pause procedure
    CALL 	delay1		; 0,65ms
    CALL 	mem_pad1	; check another paddle while pause
    NOP				; every 0,65 ms during pause cycle
    CALL 	delay1		; 0,65ms
    CALL 	mem_pad1	; check another paddle while pause
    DECFSZ 	tmr1var3,F	; check if length exhausted
    GOTO 	loop22		; cycling if not
    BTFSC 	PAD_FLAG,1	; check sign memory
    GOTO 	paddle1	        ; there is sign in memory
    GOTO 	start		; there is not 

;************************ DASH ***************************

paddle1
    BSF 	GPIO,5       	; Set OUT ON
    MOVF 	tmr1var2,W      ; load length
    MOVWF 	tmr1var3	; of 1 dash	
    BCF 	PAD_FLAG,1      ; reset memory flag
loop11
    BSF 	GPIO,4          ; Set AUDIO ON
    CALL 	delay1          ; 0,65ms : half period for 750Hz
    CALL 	mem_pad2	; check another paddle while ON
    BCF 	GPIO,4          ; Set AUDIO OFF
    CALL 	delay1          ; 0,65ms : half period for 750Hz
    CALL 	mem_pad2	; check another paddle while ON
    DECFSZ 	tmr1var3,F	; check if length exhausted
    GOTO 	loop11		; cycling if not
    BCF 	GPIO,5          ; Set OUT OFF
    MOVF 	tmr1var1,W	; load length	
    MOVWF 	tmr1var3	; of pause = 1 dot		 
    NOP
loop12
    NOP				; pause procedure
    CALL 	delay1		; 0,65ms
    CALL 	mem_pad2	; check another paddle while pause
    NOP				; every 0,65 ms during pause cycle
    CALL 	delay1		; 0,65ms
    CALL 	mem_pad2	; check another paddle while pause
    DECFSZ 	tmr1var3,F	; check if length exhausted
    GOTO 	loop12		; cycling if not
    BTFSC 	PAD_FLAG,0	; check sign memory
    GOTO 	paddle2		; there is sign in memory
    GOTO 	start		; there is not 

;*********************************************************

mem_pad1
    BTFSS 	GPIO,1         ; Check Paddle 1
    BSF 	PAD_FLAG,1

    RETURN

;*********************************************************

mem_pad2
    BTFSS 	GPIO,2         ; Check Paddle 2
    BSF 	PAD_FLAG,0

    RETURN

;*********************************************************

chk_speed 
    
    BCF 	GPIO,0          ; 
    BSF 	STATUS,RP0   	; Bank1
    BCF 	TRISIO,0        ; Set GPIO,0 as OUTPUT 
    NOP				; Keep GPIO,0 LOW for 2us
    NOP				; to discharge capasitor 
    BSF 	TRISIO,0        ; Set GPIO,0 as INPUT/start charging  
    BCF 	STATUS,RP0      ; Bank0

    BCF 	CMCON,0         ; comparator ON
    CALL 	delay2		; 14us delay to charge capasitor
    MOVF 	CMCON,F   	; moving a register to itself will set/clear Z flag
                                ; depending on the contents of that location
    BCF 	PIR1,3          ; clear comparator interrupt flag
    BSF 	T1CON,0         ; Start Timer1
loop3
    BTFSS 	PIR1,3         	; Check comparator interrupt flag
    GOTO 	loop3		; Wait for comparator interrupt
    BCF 	T1CON,0       	; Stop Timer1
    BSF 	CMCON,0       	; comparator OFF

    MOVF 	TMR1L,W         ; 
    MOVWF 	tmr1var1        ; length of 1 dot
    MOVWF 	tmr1var2	; length of 1 dash
    ADDWF 	tmr1var2,F      ; 1 dash = 1 dot + 1 dot
    ADDWF 	tmr1var2,F      ; 1 dash = 2 dots + 1 dot = 3 dots
    CLRF 	TMR1L           ; clear TMR1 register
          
    RETURN

;*********************************************************

delay1                   	;   650 cycles = 0,65 ms
    MOVLW 	0xD8           	;   d'216'
    MOVWF 	counter
loop1
    DECFSZ 	counter,F
    GOTO 	loop1

    RETURN

;*********************************************************

delay2			 	;   14 cycles = 14 us
    MOVLW 	0x03           	;   d'003'
    MOVWF 	counter
loop2
    DECFSZ 	counter,F
    GOTO 	loop2

    RETURN

;*********************************************************

    End
